summaryrefslogtreecommitdiff
path: root/lisp/vc-sccs.el
diff options
context:
space:
mode:
Diffstat (limited to 'lisp/vc-sccs.el')
-rw-r--r--lisp/vc-sccs.el127
1 files changed, 79 insertions, 48 deletions
diff --git a/lisp/vc-sccs.el b/lisp/vc-sccs.el
index 7e6cbf91279..e74d73f1014 100644
--- a/lisp/vc-sccs.el
+++ b/lisp/vc-sccs.el
@@ -27,6 +27,10 @@
;;; Commentary:
+;; TODO:
+;; - remove call to vc-expand-dirs by implementing our own (which can just
+;; list the SCCS subdir instead).
+
;;; Code:
(eval-when-compile
@@ -85,6 +89,11 @@ For a description of possible values, see `vc-check-master-templates'."
(defconst vc-sccs-name-assoc-file "VC-names")
+;;; Properties of the backend
+
+(defun vc-sccs-revision-granularity ()
+ 'file)
+
;;;
;;; State-querying functions
;;;
@@ -102,8 +111,8 @@ For a description of possible values, see `vc-check-master-templates'."
(with-temp-buffer
(if (vc-insert-file (vc-sccs-lock-file file))
(let* ((locks (vc-sccs-parse-locks))
- (workfile-version (vc-workfile-version file))
- (locking-user (cdr (assoc workfile-version locks))))
+ (working-revision (vc-working-revision file))
+ (locking-user (cdr (assoc working-revision locks))))
(if (not locking-user)
(if (vc-workfile-unchanged-p file)
'up-to-date
@@ -136,13 +145,13 @@ For a description of possible values, see `vc-check-master-templates'."
(vc-sccs-state file))))
(vc-sccs-state file)))
-(defun vc-sccs-workfile-version (file)
- "SCCS-specific version of `vc-workfile-version'."
+(defun vc-sccs-working-revision (file)
+ "SCCS-specific version of `vc-working-revision'."
(with-temp-buffer
;; The workfile version is always the latest version number.
;; To find this number, search the entire delta table,
;; rather than just the first entry, because the
;; first entry might be a deleted ("R") version.
(vc-insert-file (vc-name file) "^\001e\n\001[^s]")
(vc-parse-buffer "^\001d D \\([^ ]+\\)" 1)))
@@ -154,23 +163,29 @@ For a description of possible values, see `vc-check-master-templates'."
"SCCS-specific implementation of `vc-workfile-unchanged-p'."
(zerop (apply 'vc-do-command nil 1 "vcdiff" (vc-name file)
(list "--brief" "-q"
- (concat "-r" (vc-workfile-version file))))))
+ (concat "-r" (vc-working-revision file))))))
;;;
;;; State-changing functions
;;;
-(defun vc-sccs-register (file &optional rev comment)
- "Register FILE into the SCCS version-control system.
+(defun vc-sccs-create-repo ()
+ "Create a new SCCS repository."
+ ;; SCCS is totally file-oriented, so all we have to do is make the directory
+ (make-directory "SCCS"))
+
+(defun vc-sccs-register (files &optional rev comment)
+ "Register FILES into the SCCS version-control system.
REV is the optional revision number for the file. COMMENT can be used
-to provide an initial description of FILE.
+to provide an initial description of FILES.
`vc-register-switches' and `vc-sccs-register-switches' are passed to
the SCCS command (in that order).
-Automatically retrieve a read-only version of the file with keywords
+Automatically retrieve a read-only version of the files with keywords
expanded if `vc-keep-workfiles' is non-nil, otherwise, delete the workfile."
+ (dolist (file files)
(let* ((dirname (or (file-name-directory file) ""))
(basename (file-name-nondirectory file))
(project-file (vc-sccs-search-project-dir dirname basename)))
@@ -178,14 +193,14 @@ expanded if `vc-keep-workfiles' is non-nil, otherwise, delete the workfile."
(or project-file
(format (car vc-sccs-master-templates) dirname basename))))
(apply 'vc-do-command nil 0 "admin" vc-name
- (and rev (concat "-r" rev))
+ (and rev (not (string= rev "")) (concat "-r" rev))
"-fb"
(concat "-i" (file-relative-name file))
(and comment (concat "-y" comment))
(vc-switches 'SCCS 'register)))
(delete-file file)
(if vc-keep-workfiles
- (vc-do-command nil 0 "get" (vc-name file)))))
+ (vc-do-command nil 0 "get" (vc-name file))))))
(defun vc-sccs-responsible-p (file)
"Return non-nil if SCCS thinks it would be responsible for registering FILE."
@@ -194,16 +209,17 @@ expanded if `vc-keep-workfiles' is non-nil, otherwise, delete the workfile."
(stringp (vc-sccs-search-project-dir (or (file-name-directory file) "")
(file-name-nondirectory file)))))
-(defun vc-sccs-checkin (file rev comment)
+(defun vc-sccs-checkin (files rev comment)
"SCCS-specific version of `vc-backend-checkin'."
- (apply 'vc-do-command nil 0 "delta" (vc-name file)
- (if rev (concat "-r" rev))
- (concat "-y" comment)
- (vc-switches 'SCCS 'checkin))
- (if vc-keep-workfiles
- (vc-do-command nil 0 "get" (vc-name file))))
-
-(defun vc-sccs-find-version (file rev buffer)
+ (dolist (file files)
+ (apply 'vc-do-command nil 0 "delta" (vc-name file)
+ (if rev (concat "-r" rev))
+ (concat "-y" comment)
+ (vc-switches 'SCCS 'checkin))
+ (if vc-keep-workfiles
+ (vc-do-command nil 0 "get" (vc-name file)))))
+
+(defun vc-sccs-find-revision (file rev buffer)
(apply 'vc-do-command
buffer 0 "get" (vc-name file)
"-s" ;; suppress diagnostic output
@@ -214,7 +230,7 @@ expanded if `vc-keep-workfiles' is non-nil, otherwise, delete the workfile."
(vc-switches 'SCCS 'checkout)))
(defun vc-sccs-checkout (file &optional editable rev)
- "Retrieve a copy of a saved version of SCCS controlled FILE.
+ "Retrieve a copy of a saved revision of SCCS controlled FILE.
EDITABLE non-nil means that the file should be writable and
locked. REV is the revision to check out."
(let ((file-buffer (get-file-buffer file))
@@ -242,38 +258,52 @@ locked. REV is the revision to check out."
switches))))
(message "Checking out %s...done" file)))
+(defun vc-sccs-rollback (files)
+ "Roll back, undoing the most recent checkins of FILES."
+ (if (not files)
+ (error "SCCS backend doesn't support directory-level rollback."))
+ (dolist (file files)
+ (let ((discard (vc-working-revision file)))
+ (if (null (yes-or-no-p (format "Remove version %s from %s history? "
+ discard file)))
+ (error "Aborted"))
+ (message "Removing revision %s from %s..." discard file)
+ (vc-do-command nil 0 "rmdel" (vc-name file) (concat "-r" discard))
+ (vc-do-command nil 0 "get" (vc-name file) nil))))
+
(defun vc-sccs-revert (file &optional contents-done)
"Revert FILE to the version it was based on."
(vc-do-command nil 0 "unget" (vc-name file))
(vc-do-command nil 0 "get" (vc-name file))
- ;; Checking out explicit versions is not supported under SCCS, yet.
- ;; We always "revert" to the latest version; therefore
- ;; vc-workfile-version is cleared here so that it gets recomputed.
- (vc-file-setprop file 'vc-workfile-version nil))
-
-(defun vc-sccs-cancel-version (file editable)
- "Undo the most recent checkin of FILE.
-EDITABLE non-nil means previous version should be locked."
- (vc-do-command nil 0 "rmdel"
- (vc-name file)
- (concat "-r" (vc-workfile-version file)))
- (vc-do-command nil 0 "get"
- (vc-name file)
- (if editable "-e")))
+ ;; Checking out explicit revisions is not supported under SCCS, yet.
+ ;; We always "revert" to the latest revision; therefore
+ ;; vc-working-revision is cleared here so that it gets recomputed.
+ (vc-file-setprop file 'vc-working-revision nil))
(defun vc-sccs-steal-lock (file &optional rev)
"Steal the lock on the current workfile for FILE and revision REV."
(vc-do-command nil 0 "unget" (vc-name file) "-n" (if rev (concat "-r" rev)))
(vc-do-command nil 0 "get" (vc-name file) "-g" (if rev (concat "-r" rev))))
+(defun vc-sccs-modify-change-comment (files rev comment)
+ "Modify (actually, append to) the change comments for FILES on a specified REV."
+ (dolist (file files)
+ (vc-do-command nil 0 "cdc" (vc-name file)
+ (concat "-y" comment) (concat "-r" rev))))
+
;;;
;;; History functions
;;;
-(defun vc-sccs-print-log (file &optional buffer)
- "Get change log associated with FILE."
- (vc-do-command buffer 0 "prs" (vc-name file)))
+(defun vc-sccs-print-log (files &optional buffer)
+ "Get change log associated with FILES."
+ (vc-do-command buffer 0 "prs" (mapcar 'vc-name files)))
+
+(defun vc-sccs-wash-log ()
+ "Remove all non-comment information from log output."
+ ;; FIXME: not implemented for SCCS
+ nil)
(defun vc-sccs-logentry-check ()
"Check that the log entry in the current buffer is acceptable for SCCS."
@@ -281,11 +311,12 @@ EDITABLE non-nil means previous version should be locked."
(goto-char 512)
(error "Log must be less than 512 characters; point is now at pos 512")))
-(defun vc-sccs-diff (file &optional oldvers newvers buffer)
- "Get a difference report using SCCS between two versions of FILE."
+(defun vc-sccs-diff (files &optional oldvers newvers buffer)
+ "Get a difference report using SCCS between two filesets."
(setq oldvers (vc-sccs-lookup-triple file oldvers))
(setq newvers (vc-sccs-lookup-triple file newvers))
- (apply 'vc-do-command (or buffer "*vc-diff*") 1 "vcdiff" (vc-name file)
+ (apply 'vc-do-command (or buffer "*vc-diff*")
+ 1 "vcdiff" (mapcar 'vc-name (vc-expand-dirs files))
(append (list "-q"
(and oldvers (concat "-r" oldvers))
(and newvers (concat "-r" newvers)))
@@ -297,8 +328,8 @@ EDITABLE non-nil means previous version should be locked."
;;;
(defun vc-sccs-assign-name (file name)
- "Assign to FILE's latest version a given NAME."
- (vc-sccs-add-triple name file (vc-workfile-version file)))
+ "Assign to FILE's latest revision a given NAME."
+ (vc-sccs-add-triple name file (vc-working-revision file)))
;;;
@@ -363,7 +394,7 @@ find any project directory."
(defun vc-sccs-parse-locks ()
"Parse SCCS locks in current buffer.
-The result is a list of the form ((VERSION . USER) (VERSION . USER) ...)."
+The result is a list of the form ((REVISION . USER) (REVISION . USER) ...)."
(let (master-locks)
(goto-char (point-min))
(while (re-search-forward "^\\([0-9.]+\\) [0-9.]+ \\([^ ]+\\) .*\n?"
@@ -384,8 +415,8 @@ The result is a list of the form ((VERSION . USER) (VERSION . USER) ...)."
(kill-buffer (current-buffer))))
(defun vc-sccs-lookup-triple (file name)
- "Return the numeric version corresponding to a named snapshot of FILE.
-If NAME is nil or a version number string it's just passed through."
+ "Return the numeric revision corresponding to a named snapshot of FILE.
+If NAME is nil or a revision number string it's just passed through."
(if (or (null name)
(let ((firstchar (aref name 0)))
(and (>= firstchar ?0) (<= firstchar ?9))))
@@ -136,13 +145,13 @@ For a description of possible values, see `vc-check-master-templates'."
(with-temp-buffer
- ;; The workfile version is always the latest version number.
+ ;; The working revision is always the latest revision number.
;; To find this number, search the entire delta table,
;; rather than just the first entry, because the
@@ -136,13 +145,13 @@ For a description of possible values, see `vc-check-master-templates'."
;; To find this number, search the entire delta table,
;; rather than just the first entry, because the
- ;; first entry might be a deleted ("R") version.
+ ;; first entry might be a deleted ("R") revision.
(vc-insert-file (vc-name file) "^\001e\n\001[^s]")
(vc-parse-buffer "^\001d D \\([^ ]+\\)" 1)))